import arg from "arg";
import inquirer from "inquirer";
import changeCase from "change-case";

import createComponent from "./main";

function parseArgumentsIntoOptions(rawArgs) {
  const args = arg(
    {
      "--yes": Boolean,
      "-y": "--yes"
    },
    {
      argv: rawArgs.slice(2)
    }
  );
  return {
    skipPrompts: args["--yes"] || false,
    template: args._[0],
    componentName: args._[1]
  };
}

async function promptForMissingOptions(options) {
  const defaultTemplate = {
    template: "Functional",
    componentName: "Index"
  };
  if (options.skipPrompts) {
    return {
      ...options,
      template: options.template || defaultTemplate.template,
      componentName: options.componentName || defaultTemplate.componentName,
      directoryName: options.componentName || defaultTemplate.componentName
    };
  }

  const questions = [];
  if (!options.template) {
    questions.push({
      type: "list",
      name: "template",
      message: "Please choose which template to use",
      choices: ["Functional", "Class"],
      default: defaultTemplate
    });
  }

  if (!options.componentName) {
    questions.push({
      type: "input",
      name: "componentName",
      message: "Please enter component name",
      default: "Index"
    });
  }

  const answers = await inquirer.prompt(questions);
  return {
    ...options,
    template: options.template || answers.template,
    componentName: options.componentName || answers.componentName,
    directoryName: options.componentName || answers.componentName
  };
}

function changeOptionsCases(options) {
  return {
    ...options,
    template: changeCase.lowerCase(options.template),
    componentName: changeCase.pascalCase(options.componentName),
    directoryName: changeCase.kebabCase(options.componentName)
  };
}

export async function cli(args) {
  let options = parseArgumentsIntoOptions(args);
  options = await promptForMissingOptions(options);
  options = changeOptionsCases(options);
  await createComponent(options);
}
