import chalk from "chalk";
import fs from "fs";
import ncp from "ncp";
import path from "path";
import { promisify } from "util";
import changeCase from "change-case";

const copy = promisify(ncp);

function createComponentDirectory(options) {
  const componentDirectory = path.resolve(process.cwd(), options.directoryName);

  if (!fs.existsSync(componentDirectory)) {
    fs.mkdirSync(componentDirectory);
  }

  return componentDirectory;
}

function getTemplateDirectory(options) {
  const currentFileUrl = import.meta.url;
  return path.resolve(
    new URL(currentFileUrl).pathname,
    "../../templates",
    options.template
  );
}

async function copyTemplateFiles(options) {
  return copy(options.templateDirectory, options.targetDirectory, {
    clobber: false
  });
}

function readWriteAsync(options) {
  fs.readFile(`${options.targetDirectory}/index.tsx`, "utf-8", function(
    err,
    data
  ) {
    if (err) {
      throw err;
    }

    var newValue = data.replace(/Index/gm, options.componentName);

    fs.writeFile(
      `${options.targetDirectory}/${options.componentName}.tsx`,
      newValue,
      "utf-8",
      function(err) {
        if (err) {
          throw err;
        }
      }
    );
  });
}

function deleteTemplate(options) {
  fs.unlinkSync(`${options.targetDirectory}/index.tsx`);
}

async function createComponent(options) {
  options = {
    ...options,
    targetDirectory: createComponentDirectory(options),
    templateDirectory: getTemplateDirectory(options)
  };

  await copyTemplateFiles(options);
  readWriteAsync(options);
  deleteTemplate(options);

  console.log("%s Component is ready", chalk.green.bold("DONE"));
  return true;
}

export default createComponent;
